@@ -169,10 +169,10 @@ def _configure_logging(verbosity=0):
         level = settings.DEFAULT_LOGGING_LEVEL
         default_format = settings.DEFAULT_LOGGING_FORMAT
         verbose_format = settings.VERBOSE_LOGGING_FORMAT
-    elif verbosity == 1:
+    elif verbosity in (1, 2):  # level 2 adds verbose string formatting
         level = settings.VERBOSE_LOGGING_LEVEL
         default_format = verbose_format = settings.VERBOSE_LOGGING_FORMAT
-    elif verbosity == 2:
+    elif verbosity == 3:
         level = settings.VERBOSE2_LOGGING_LEVEL
         default_format = verbose_format = settings.VERBOSE_LOGGING_FORMAT
     else:
@@ -185,6 +185,11 @@ def _configure_logging(verbosity=0):
     formatter = _WarningFormatter(default_format, verbose_format)
     logging.root.handlers[0].setFormatter(formatter)
 
+    # Warn about excessive verbosity
+    if common.VERBOSITY > 4:
+        logging.warn("maximum supported verbosity level is 4")
+        common.VERBOSITY = 4
+
 
 def _run(args, cwd, err):  # pylint: disable=W0613
     """Process arguments and run the `doorstop` subcommand.
@@ -194,14 +199,14 @@ def _run(args, cwd, err):  # pylint: disable=W0613
     """
     try:
         tree = build(cwd)
-        tree.check()
+        valid = tree.valid()
     except DoorstopError as error:
         logging.error(error)
         return False
     else:
-        if tree:
+        if tree and valid:
             print("valid tree: {}".format(tree))
-        return True
+        return valid
 
 
 def _run_new(args, cwd, _):
@@ -358,7 +363,7 @@ def _run_publish(args, cwd, _):
 
         if args.path:
             print("publishing {} to {}...".format(document, args.path))
-            report.publish(document, args.path, ext)
+            report.publish(document, args.path, ext, **kwargs)
         else:
             for line in report.iter_lines(document, ext, **kwargs):
                 print(line)
