@@ -7,10 +7,10 @@ import logging
 
 import yaml
 
-from doorstop.core.item import Item
+from doorstop.core.item import Item, split_id
 from doorstop import common
-from doorstop.common import DoorstopError
-from doorstop import settings
+from doorstop.common import DoorstopError, DoorstopWarning, DoorstopInfo
+from doorstop.settings import SEP_CHARS
 
 
 class Document(object):
@@ -33,7 +33,7 @@ class Document(object):
         @param path: path to Document directory
         @param root: path to root of project
         """
-        # Check document's directory
+        # Ensure the directory is valid
         if not os.path.isfile(os.path.join(path, Document.CONFIG)):
             relpath = os.path.relpath(path, root)
             msg = "no {} in {}".format(Document.CONFIG, relpath)
@@ -74,20 +74,20 @@ class Document(object):
         return not self == other
 
     @staticmethod
-    def new(path, root, prefix, sep=None, parent=None, digits=None):
+    def new(path, root, prefix, sep=None, parent=None, digits=None):  # pylint: disable=R0913
         """Create a new Document.
 
         @param path: path to directory for the new document
         @param root: path to root of the project
         @param prefix: prefix for the new document
-        @param sep: separator ('_', '-', or '.') between prefix and numbers
+        @param sep: separator between prefix and numbers
         @param parent: parent ID for the new document
         @param digits: number of digits for the new document
 
         @raise DoorstopError: if the document already exists
         """
-        # TODO: remove after testing or raise specific exception
-        assert sep is None or sep in settings.SEP_CHARS
+        # TODO: remove after testing or raise a specific exception
+        assert sep is None or sep in SEP_CHARS
         config = os.path.join(path, Document.CONFIG)
         # Check for an existing document
         if os.path.exists(config):
@@ -203,21 +203,70 @@ class Document(object):
         return Item.new(self.path, self.root, self.prefix, self.sep, number,
                         self.digits, level)
 
-    def check(self, tree=None, ignored=None):
-        """Confirm the document is valid.
+    def find_item(self, identifier, _kind=''):
+        """Return an item from its ID.
+
+        @param identifier: item ID
+
+        @return: matching Item
+
+        @raise DoorstopError: if the item cannot be found
+        """
+        # Search using the prefix and number
+        prefix, number = split_id(identifier)
+        if self.prefix.lower() == prefix.lower():
+            for item in self:
+                if item.number == number:
+                    return item
+            msg = "no matching{} number: {}".format(_kind, number)
+            logging.debug(msg)
+
+        # Fall back to a search using the exact ID
+        else:
+            for item in self:
+                if item.id.lower() == identifier.lower():
+                    return item
+
+        raise DoorstopError("no matching{} ID: {}".format(_kind, identifier))
+
+    def valid(self, tree=None, ignored=None):
+        """Check the document (and its items) for validity.
 
         @param tree: tree to validate the document
         @param ignored: function to determine if a path should be skipped
 
         @return: indication that document is valid
         """
+        valid = True
+        # Display all issues
+        for issue in self.iter_issues(tree=tree, ignored=ignored):
+            if isinstance(issue, DoorstopInfo):
+                logging.info(issue)
+            elif isinstance(issue, DoorstopWarning):
+                logging.warning(issue)
+            else:
+                assert isinstance(issue, DoorstopError)
+                logging.error(issue)
+                valid = False
+        # Return the result
+        return valid
+
+    def iter_issues(self, tree=None, ignored=None):
+        """Yield all the document's issues.
+
+        @param tree: tree to validate the item
+        @param ignored: function to determine if a path should be skipped
+
+        @return: generator of DoorstopError, DoorstopWarning, DoorstopInfo
+        """
         logging.info("checking document {}...".format(self))
         items = list(self)
         # Check for items
         if not items:
-            logging.warning("no items: {}".format(self))
+            yield DoorstopWarning("no items")
         # Check each item
         for item in items:
-            item.check(document=self, tree=tree, ignored=ignored)
-        # Document is valid
-        return True
+            for issue in item.iter_issues(document=self, tree=tree,
+                                          ignored=ignored):
+                # Prepend the item's ID
+                yield type(issue)("{}: {}".format(item.id, issue))
