@@ -10,8 +10,8 @@ import logging
 import yaml
 
 from doorstop import common
-from doorstop.common import DoorstopError
-from doorstop import settings
+from doorstop.common import DoorstopError, DoorstopWarning, DoorstopInfo
+from doorstop.settings import SEP_CHARS
 
 
 def auto_load(func):
@@ -37,7 +37,7 @@ def auto_save(func):
     return wrapped
 
 
-class Item(object):  # pylint: disable=R0902,R0904
+class Item(object):  # pylint: disable=R0904
     """Represents a file with linkable text that is part of a document."""
 
     EXTENSIONS = '.yml', '.yaml'
@@ -61,10 +61,10 @@ class Item(object):  # pylint: disable=R0902,R0904
         @param path: path to Item file
         @param root: path to root of project
         """
-        # Check item's path
+        # Ensure the path is valid
         if not os.path.isfile(path):
             raise DoorstopError("item does not exist: {}".format(path))
-        # Check file name
+        # Ensure the filename is valid
         filename = os.path.basename(path)
         name, ext = os.path.splitext(filename)
         try:
@@ -72,7 +72,7 @@ class Item(object):  # pylint: disable=R0902,R0904
         except DoorstopError:
             msg = "invalid item filename: {}".format(filename)
             raise DoorstopError(msg) from None
-        # Check file extension
+        # Ensure the file extension is valid
         if ext.lower() not in self.EXTENSIONS:
             msg = "'{0}' extension not in {1}".format(path, self.EXTENSIONS)
             raise DoorstopError(msg)
@@ -80,14 +80,14 @@ class Item(object):  # pylint: disable=R0902,R0904
         self.path = path
         self.root = root
         self._exists = True
-        self._level = Item.DEFAULT_LEVEL
-        self._active = Item.DEFAULT_ACTIVE
-        self._normative = Item.DEFAULT_NORMATIVE
-        self._derived = Item.DEFAULT_DERIVED
-        self._text = Item.DEFAULT_TEXT
-        self._ref = Item.DEFAULT_REF
-        self._links = Item.DEFAULT_LINKS
         self._data = {}
+        self._data['level'] = Item.DEFAULT_LEVEL
+        self._data['active'] = Item.DEFAULT_ACTIVE
+        self._data['normative'] = Item.DEFAULT_NORMATIVE
+        self._data['derived'] = Item.DEFAULT_DERIVED
+        self._data['text'] = Item.DEFAULT_TEXT
+        self._data['ref'] = Item.DEFAULT_REF
+        self._data['links'] = Item.DEFAULT_LINKS
 
     def __repr__(self):
         return "Item({})".format(repr(self.path))
@@ -156,18 +156,28 @@ class Item(object):  # pylint: disable=R0902,R0904
         text = self._read()
         # Parse the YAML data
         try:
-            self._data = yaml.load(text) or {}
+            data = yaml.load(text) or {}
         except yaml.scanner.ScannerError as error:  # pylint: disable=E1101
             msg = "invalid contents: {}:\n{}".format(self, error)
             raise DoorstopError(msg)
         # Store parsed data
-        self._level = convert_level(self._data.get('level', self._level))
-        self._active = bool(self._data.get('active', self._active))
-        self._normative = bool(self._data.get('normative', self._normative))
-        self._derived = bool(self._data.get('derived', self._derived))
-        self._text = self._data.get('text', self._text).strip()
-        self._ref = self._data.get('ref', self._ref)
-        self._links = set(self._data.get('links', self._links))
+        for key, value in data.items():
+            if key == 'level':
+                self._data['level'] = convert_level(value)
+            elif key == 'active':
+                self._data['active'] = bool(value)
+            elif key == 'normative':
+                self._data['normative'] = bool(value)
+            elif key == 'derived':
+                self._data['derived'] = bool(value)
+            elif key == 'text':
+                self._data['text'] = value.strip()
+            elif key == 'ref':
+                self._data['ref'] = value.strip()
+            elif key == 'links':
+                self._data['links'] = set(value)
+            else:
+                self._data[key] = value
         # Set meta attributes
         setattr(self, '_loaded', True)
 
@@ -181,26 +191,24 @@ class Item(object):  # pylint: disable=R0902,R0904
     def save(self):
         """Format and save the item's properties to a file."""
         logging.debug("saving {}...".format(repr(self)))
-        # Collect the data items
-        level = '.'.join(str(n) for n in self._level)
-        if len(self._level) == 2:
-            level = float(level)
-        text = Literal(sbd(self._text))
-        active = self._active
-        normative = self._normative
-        derived = self._derived
-        ref = self._ref.strip()
-        links = sorted(self._links)
-        # Build the data structure
-        self._data['level'] = level
-        self._data['normative'] = normative
-        self._data['derived'] = derived
-        self._data['active'] = active
-        self._data['text'] = text
-        self._data['links'] = links
-        self._data['ref'] = ref
+        # Format the data items
+        data = {}
+        for key, value in self._data.items():
+            if key == 'level':
+                level = '.'.join(str(n) for n in value)
+                if len(value) == 2:
+                    level = float(level)
+                data['level'] = level
+            elif key == 'text':
+                data['text'] = Literal(sbd(self._data['text']))
+            elif key == 'ref':
+                data['ref'] = value.strip()
+            elif key == 'links':
+                data['links'] = sorted(value)
+            else:
+                data[key] = value
         # Dump the data to YAML
-        dump = yaml.dump(self._data, default_flow_style=False)
+        dump = yaml.dump(data, default_flow_style=False)
         # Save the YAML to file
         self._write(dump)
         # Set meta attributes
@@ -247,13 +255,13 @@ class Item(object):  # pylint: disable=R0902,R0904
     @auto_load
     def level(self):
         """Get the item level."""
-        return self._level
+        return self._data['level']
 
     @level.setter
     @auto_save
     def level(self, level):
         """Set the item's level."""
-        self._level = convert_level(level)
+        self._data['level'] = convert_level(level)
 
     @property
     def depth(self):
@@ -274,13 +282,13 @@ class Item(object):  # pylint: disable=R0902,R0904
          - externally implemented requirements
 
         """
-        return self._active
+        return self._data['active']
 
     @active.setter
     @auto_save
     def active(self, status):
         """Set the item's active status."""
-        self._active = bool(status)
+        self._data['active'] = bool(status)
 
     @property
     @auto_load
@@ -293,13 +301,13 @@ class Item(object):  # pylint: disable=R0902,R0904
          - comments
 
         """
-        return self._normative
+        return self._data['normative']
 
     @normative.setter
     @auto_save
     def normative(self, status):
         """Set the item's normative status."""
-        self._normative = bool(status)
+        self._data['normative'] = bool(status)
 
     @property
     @auto_load
@@ -310,13 +318,13 @@ class Item(object):  # pylint: disable=R0902,R0904
         document, but should still be linked to by items in its child
         document.
         """
-        return self._derived
+        return self._data['derived']
 
     @derived.setter
     @auto_save
     def derived(self, status):
         """Set the item's derived status."""
-        self._derived = bool(status)
+        self._data['derived'] = bool(status)
 
     @property
     def header(self):
@@ -327,50 +335,62 @@ class Item(object):  # pylint: disable=R0902,R0904
     @auto_load
     def text(self):
         """Get the item's text."""
-        return self._text
+        return self._data['text']
 
     @text.setter
     @auto_save
     def text(self, text):
         """Set the item's text."""
-        self._text = text
+        self._data['text'] = text
 
     @property
     @auto_load
     def ref(self):
         """Get the item's external file reference."""
-        return self._ref
+        return self._data['ref']
 
     @ref.setter
     @auto_save
     def ref(self, ref):
         """Set the item's external file reference."""
-        self._ref = ref
+        self._data['ref'] = ref
 
     @property
     @auto_load
     def links(self):
         """Get the items this item links to."""
-        return sorted(self._links)
+        return sorted(self._data['links'])
 
     @links.setter
     @auto_save
     def links(self, links):
         """Set the items this item links to."""
-        self._links = set(links)
+        self._data['links'] = set(links)
 
     # extended attributes ####################################################
 
     @auto_load
     def get(self, name, default=None):
         """Get an extended attribute."""
-        return self._data.get(name, default)
+        if hasattr(self, name):
+            cname = self.__class__.__name__
+            msg = "'{n}' can be accessed from {c}.{n}".format(n=name, c=cname)
+            logging.info(msg)
+            return getattr(self, name)
+        else:
+            return self._data.get(name, default)
 
     @auto_load
     @auto_save
     def set(self, name, value):
-        """SEt an extended attribute."""
-        self._data[name] = value
+        """Set an extended attribute."""
+        if hasattr(self, name):
+            cname = self.__class__.__name__
+            msg = "'{n}' can be set from {c}.{n}".format(n=name, c=cname)
+            logging.info(msg)
+            return setattr(self, name, value)
+        else:
+            self._data[name] = value
 
     # actions ################################################################
 
@@ -378,19 +398,19 @@ class Item(object):  # pylint: disable=R0902,R0904
     @auto_save
     def add_link(self, item):
         """Add a new link to another item."""
-        self._links.add(item)
+        self._data['links'].add(item)
 
     @auto_load
     @auto_save
     def remove_link(self, item):
         """Remove an existing link."""
         try:
-            self._links.remove(item)
+            self._data['links'].remove(item)
         except KeyError:
             logging.warning("link to {0} does not exist".format(item))
 
-    def check(self, document=None, tree=None, ignored=None):
-        """Confirm the item is valid.
+    def valid(self, document=None, tree=None, ignored=None):
+        """Check the item for validity.
 
         @param document: document to validate the item
         @param tree: tree to validate the item
@@ -398,65 +418,97 @@ class Item(object):  # pylint: disable=R0902,R0904
 
         @return: indication that the item is valid
         """
+        valid = True
+        # Display all issues
+        for issue in self.iter_issues(document=document, tree=tree,
+                                      ignored=ignored):
+            if isinstance(issue, DoorstopInfo):
+                logging.info(issue)
+            elif isinstance(issue, DoorstopWarning):
+                logging.warning(issue)
+            else:
+                assert isinstance(issue, DoorstopError)
+                logging.error(issue)
+                valid = False
+        # Return the result
+        return valid
+
+    def iter_issues(self, document=None, tree=None, ignored=None):
+        """Yield all the item's issues.
+
+        @param document: document to validate the item
+        @param tree: tree to validate the item
+        @param ignored: function to determine if a path should be skipped
+
+        @return: generator of DoorstopError, DoorstopWarning, DoorstopInfo
+        """
         logging.info("checking item {}...".format(self))
         # Verify the file can be parsed
         self.load()
         # Skip inactive items
         if not self.active:
             logging.info("skipped inactive item: {}".format(self))
-            return True
+            return
+        self.auto = False
         # Check text
         if not self.text and not self.ref:
-            logging.warning("no text: {}".format(self))
+            yield DoorstopWarning("no text")
         # Check external references
-        self.find_ref(ignored=ignored)
+        try:
+            self.find_ref(ignored=ignored)
+        except DoorstopError as exc:
+            yield exc
         # Check links
         if not self.normative and self.links:
-            logging.warning("non-normative item has links: {}".format(self))
+            yield DoorstopWarning("non-normative, but has links")
         # Check links against the document
         if document:
-            self._check_document(document)
+            yield from self._iter_issues_document(document)
         # Check links against the tree
         if tree:
-            self._check_tree(tree)
+            yield from self._iter_issues_tree(tree)
         # Reformat the file
         self.save()
-        # Item is valid
-        return True
 
-    def _check_document(self, document):
-        """Check the item against its document."""
+    def _iter_issues_document(self, document):
+        """Yield all the item's issues against its document."""
         # Verify an item has upward links
         if all((document.parent,
                 self.normative,
                 not self.derived)) and not self.links:
-            logging.warning("no links: {}".format(self))
+            msg = "no links to parent document: {}".format(document.parent)
+            yield DoorstopWarning(msg)
         # Verify an item's links are to the correct parent
         for identifier in self.links:
             prefix = split_id(identifier)[0]
             if prefix.lower() != document.parent.lower():
-                msg = "link to non-parent '{}' in {}".format(identifier, self)
-                logging.info(msg)
+                msg = "linked to non-parent item: {}".format(identifier)
+                yield DoorstopInfo(msg)
 
-    def _check_tree(self, tree):
-        """Check the item against the full tree."""
+    def _iter_issues_tree(self, tree):
+        """Yield all the item's issues against the full tree."""
         # Verify an item's links are valid
         identifiers = []
         for identifier in self.links:
             try:
                 item = tree.find_item(identifier)
             except DoorstopError:
-                msg = "{} linked to unknown item: {}".format(self, identifier)
-                raise DoorstopError(msg) from None
-            # TODO: not sure if we want this warning
-            # if not item.active:
-            #    logging.warn("{} linked to inactive item: {}".format(self,
-            #                                                         item))
-            identifier = item.id  # re-format the item's ID
-            logging.debug("found linked item: {}".format(identifier))
-            identifiers.append(identifier)
-        self._links = set(identifiers)
-        # Verify an item has downward (reverse) links
+                identifiers.append(identifier)  # keep the invalid ID
+                msg = "linked to unknown item: {}".format(identifier)
+                yield DoorstopError(msg)
+            else:
+                if not item.active:
+                    msg = "linked to inactive item: {}".format(item)
+                    yield DoorstopInfo(msg)
+                if not item.normative:
+                    msg = "linked to non-normative item: {}".format(item)
+                    yield DoorstopWarning(msg)
+                identifier = item.id  # reformat the item's ID
+                logging.debug("found linked item: {}".format(identifier))
+                identifiers.append(identifier)
+        # Apply the reformatted item IDs
+        self._data['links'] = set(identifiers)
+        # Verify an item is being linked to (reverse links)
         rlinks = []
         children = []
         for document in tree:
@@ -466,18 +518,12 @@ class Item(object):  # pylint: disable=R0902,R0904
                     if self.id in item.links:
                         rlinks.append(item.id)
         if rlinks:
-            if self.normative:
-                msg = "down links: {}".format(', '.join(rlinks))
-                logging.debug(msg)
-            else:
-                for rlink in rlinks:
-                    msg = "{} links to non-normative {}".format(rlink,
-                                                                self)
-                    logging.warning(msg)
+            msg = "reverse links: {}".format(', '.join(rlinks))
+            logging.debug(msg)
         elif self.normative:
             for child in children:
-                msg = "{} has no links from {}".format(self, child)
-                logging.warning(msg)
+                msg = "no links from child document: {}".format(child)
+                yield DoorstopWarning(msg)
 
     def find_ref(self, root=None, ignored=None):
         """Find the external file reference and line number.
@@ -495,7 +541,7 @@ class Item(object):  # pylint: disable=R0902,R0904
             logging.debug("no external reference to search for")
             return None, None
         ignored = ignored or (lambda _: False)
-        logging.info("seraching for ref '{}'...".format(self.ref))
+        logging.debug("seraching for ref '{}'...".format(self.ref))
         pattern = r"(\b|\W){}(\b|\W)".format(re.escape(self.ref))
         logging.debug("regex: {}".format(pattern))
         regex = re.compile(pattern)
@@ -519,7 +565,7 @@ class Item(object):  # pylint: disable=R0902,R0904
                     with open(path, 'r') as external:
                         for index, line in enumerate(external):
                             if regex.search(line):
-                                logging.info("found ref: {}".format(relpath))
+                                logging.debug("found ref: {}".format(relpath))
                                 return relpath, index + 1
                 except UnicodeDecodeError:
                     pass
@@ -562,7 +608,7 @@ def split_id(text):
     match = re.match(r"([\w.-]*\D)(\d+)", text)
     if not match:
         raise DoorstopError("invalid ID: {}".format(text))
-    prefix = match.group(1).rstrip(settings.SEP_CHARS)
+    prefix = match.group(1).rstrip(SEP_CHARS)
     number = int(match.group(2))
     return prefix, number
 
