@@ -13,7 +13,7 @@ import logging
 from doorstop.core.item import Item
 from doorstop.core.document import Document
 from doorstop import common
-from doorstop.common import DoorstopError
+from doorstop.common import DoorstopError, DoorstopWarning, DoorstopInfo
 
 from doorstop.core.test import ENV, REASON, ROOT, FILES, EMPTY, NEW
 
@@ -160,16 +160,43 @@ class TestDocument(unittest.TestCase):  # pylint: disable=R0904
         self.assertIsNot(None, document.add())
         mock_new.assert_called_once_with(NEW, ROOT, 'NEW', '', 1, 5, None)
 
-    @patch('doorstop.core.item.Item.check')
-    def test_check(self, mock_check):
+    def test_find_item(self):
+        """Verify an item can be found by ID."""
+        item = self.document.find_item('req2')
+        self.assertIsNot(None, item)
+
+    def test_find_item_exact(self):
+        """Verify an item can be found by its exact ID."""
+        item = self.document.find_item('req2-001')
+        self.assertIsNot(None, item)
+
+    def test_find_item_unknown_number(self):
+        """Verify an exception is raised on an unknown number."""
+        self.assertRaises(DoorstopError, self.document.find_item, 'req99')
+
+    def test_find_item_unknown_ID(self):
+        """Verify an exception is raised on an unknown ID."""
+        self.assertRaises(DoorstopError, self.document.find_item, 'unknown99')
+
+    @patch('doorstop.core.item.Item.iter_issues')
+    def test_valid(self, mock_iter_issues):
         """Verify a document can be validated."""
-        self.document.check()
-        self.assertEqual(4, mock_check.call_count)
+        mock_iter_issues.return_value = [DoorstopInfo('i')]
+        self.assertTrue(self.document.valid())
+        self.assertEqual(4, mock_iter_issues.call_count)
 
     @unittest.skipUnless(os.getenv(ENV), REASON)
-    def test_check_long(self):
+    def test_valid_long(self):
         """Verify a document can be validated (long)."""
-        self.document.check()
+        self.assertTrue(self.document.valid())
+
+    def test_valid_item(self):
+        """Verify an item error fails the document check."""
+        with patch.object(self.document, 'iter_issues',
+                          Mock(return_value=[DoorstopError('e'),
+                                             DoorstopWarning('w'),
+                                             DoorstopInfo('i')])):
+            self.assertFalse(self.document.valid())
 
 
 class TestModule(unittest.TestCase):  # pylint: disable=R0904
