@@ -9,7 +9,7 @@ import subprocess
 import logging
 from itertools import chain
 
-from doorstop.common import DoorstopError
+from doorstop.common import DoorstopError, DoorstopWarning, DoorstopInfo
 from doorstop.core.document import Document
 from doorstop.core.item import split_id
 from doorstop.core import vcs
@@ -59,39 +59,42 @@ class Tree(object):
         yield from chain(*(iter(c) for c in self.children))
 
     @staticmethod
-    def from_list(docs, root=None):
-        """Get a new tree from the list of Documents.
+    def from_list(documents, root=None):
+        """Get a new tree from the list of documents.
 
-        @param root: path to root of the project
         @param documents: list of Documents
+        @param root: path to root of the project
 
         @return: new tree
 
         @raise DoorstopError: when the tree cannot be built
         """
-        if not docs:
+        if not documents:
             return Tree(document=None, root=root)
-        unplaced = list(docs)
-        for doc in list(unplaced):
-            if doc.parent is None:
-                logging.debug("added root of tree: {}".format(doc))
-                tree = Tree(doc)
-                logging.info("root of tree: {}".format(doc))
-                unplaced.remove(doc)
+        unplaced = list(documents)
+        for document in list(unplaced):
+            if document.parent is None:
+                logging.debug("added root of tree: {}".format(document))
+                tree = Tree(document)
+                logging.info("root of the tree: {}".format(document))
+                unplaced.remove(document)
                 break
         else:
             raise DoorstopError("no root document")
 
         while unplaced:
             count = len(unplaced)
-            for doc in list(unplaced):
+            for document in list(unplaced):
+                if document.parent is None:
+                    logging.info("root of the tree: {}".format(document))
+                    raise DoorstopError("multiple root documents")
                 try:
-                    tree._place(doc)  # pylint: disable=W0212
+                    tree._place(document)  # pylint: disable=W0212
                 except DoorstopError as error:
                     logging.debug(error)
                 else:
-                    logging.info("added to tree: {}".format(doc))
-                    unplaced.remove(doc)
+                    logging.info("added to tree: {}".format(document))
+                    unplaced.remove(document)
 
             if len(unplaced) == count:  # no more documents could be placed
                 logging.debug("unplaced documents: {}".format(unplaced))
@@ -100,26 +103,28 @@ class Tree(object):
 
         return tree
 
-    def _place(self, doc):
+    def _place(self, document):
         """Attempt to place the Document in the current tree.
 
-        @param doc: Document to add
+        @param document: Document to add
 
         @raise DoorstopError: if the Document cannot yet be placed
         """
-        logging.debug("trying to add '{}'...".format(doc))
+        logging.debug("trying to add '{}'...".format(document))
         if not self.document:
 
             # Tree is empty
-            if doc.parent:
-                msg = "unknown parent for {}: {}".format(doc, doc.parent)
+            if document.parent:
+                msg = "unknown parent for {}: {}".format(document,
+                                                         document.parent)
                 raise DoorstopError(msg)
-            self.document = doc
+            self.document = document
 
-        elif doc.parent.lower() == self.document.prefix.lower():
+        elif (document.parent and
+              document.parent.lower() == self.document.prefix.lower()):
 
             # Current document is the parent
-            node = Tree(doc, self)
+            node = Tree(document, self)
             self.children.append(node)
 
         else:
@@ -127,13 +132,14 @@ class Tree(object):
             # Search for the parent
             for child in self.children:
                 try:
-                    child._place(doc)  # pylint: disable=W0212
+                    child._place(document)  # pylint: disable=W0212
                 except DoorstopError:
                     pass  # the error is raised later
                 else:
                     break
             else:
-                msg = "unknown parent for {}: {}".format(doc, doc.parent)
+                msg = "unknown parent for {}: {}".format(document,
+                                                         document.parent)
                 raise DoorstopError(msg)
 
     # attributes #############################################################
@@ -147,12 +153,12 @@ class Tree(object):
 
     # actions ################################################################
 
-    def new(self, path, prefix, sep=None, parent=None, digits=None):
+    def new(self, path, prefix, sep=None, parent=None, digits=None):  # pylint: disable=R0913
         """Create a new document and add it to the tree.
 
         @param path: directory path for the new document
         @param prefix: document's prefix
-        @param sep: separator between prefix and number for items
+        @param sep: separator between prefix and numbers
         @param parent: parent document's prefix
         @param digits: number of digits for the document's numbers
 
@@ -211,9 +217,9 @@ class Tree(object):
         """
         logging.info("linking {} to {}...".format(cid, pid))
         # Find child item
-        child = self.find_item(cid, 'child')
+        child = self.find_item(cid, _kind='child')
         # Find parent item
-        parent = self.find_item(pid, 'parent')
+        parent = self.find_item(pid, _kind='parent')
         # Add link
         child.add_link(parent.id)
         return child, parent
@@ -230,9 +236,9 @@ class Tree(object):
         """
         logging.info("unlinking {} from {}...".format(cid, pid))
         # Find child item
-        child = self.find_item(cid, 'child')
+        child = self.find_item(cid, _kind='child')
         # Find parent item
-        parent = self.find_item(pid, 'parent')
+        parent = self.find_item(pid, _kind='parent')
         # Remove link
         child.remove_link(parent.id)
         return child, parent
@@ -272,30 +278,24 @@ class Tree(object):
 
         raise DoorstopError("no matching prefix: {}".format(prefix))
 
-    def find_item(self, identifier, kind=''):
+    def find_item(self, identifier, _kind=''):
         """Return an item from its ID.
 
         @param identifier: item ID
-        @param kind: type of item for logging messages
 
         @return: matching Item
 
         @raise DoorstopError: if the item cannot be found
         """
-        _kind = (' ' + kind) if kind else kind
+        # Type of item ('parent', 'child', or '') for logging messages
+        _kind = (' ' + _kind) if _kind else _kind
 
         # Search using the prefix and number
-        prefix, number = split_id(identifier)
+        prefix = split_id(identifier)[0]
         for document in self:
             if document.prefix.lower() == prefix.lower():
-                for item in document:
-                    if item.number == number:
-                        return item
-                msg = "no matching{} number: {}".format(_kind, number)
-                logging.info(msg)
-                break
-        else:
-            logging.info("no matching{} prefix: {}".format(_kind, prefix))
+                return document.find_item(identifier, _kind=_kind)
+        logging.debug("no matching{} prefix: {}".format(_kind, prefix))
 
         # Fall back to a search using the exact ID
         for document in self:
@@ -305,17 +305,41 @@ class Tree(object):
 
         raise DoorstopError("no matching{} ID: {}".format(_kind, identifier))
 
-    def check(self):
-        """Confirm the document hiearchy is valid.
+    def valid(self):
+        """Check the tree (and its documents) for validity.
 
-        @return: indication that hiearchy is valid
-
-        @raise DoorstopError: on issue
+        @return: indication that the tree is valid
         """
+        valid = True
         logging.info("checking tree...")
-        for document in self:
-            document.check(tree=self, ignored=self.vcs.ignored)
-        return True
+        # Display all issues
+        for issue in self.iter_issues():
+            if isinstance(issue, DoorstopInfo):
+                logging.info(issue)
+            elif isinstance(issue, DoorstopWarning):
+                logging.warning(issue)
+            else:
+                assert isinstance(issue, DoorstopError)
+                logging.error(issue)
+                valid = False
+        # Return the result
+        return valid
+
+    def iter_issues(self):
+        """Yield all the tree's issues.
+
+        @return: generator of DoorstopError, DoorstopWarning, DoorstopInfo
+        """
+        documents = list(self)
+        # Check for documents
+        if not documents:
+            yield DoorstopWarning("no documents")
+        # Check each document
+        for document in documents:
+            for issue in document.iter_issues(tree=self,
+                                              ignored=self.vcs.ignored):
+                # Prepend the document's prefix
+                yield type(issue)("{}: {}".format(document.prefix, issue))
 
 
 def _open(path, tool=None):  # pragma: no cover, integration test
@@ -363,7 +387,7 @@ def build(cwd=None, root=None):
 
     # Build the tree
     if not documents:
-        logging.warning("no documents found in: {}".format(root))
+        logging.info("no documents found in: {}".format(root))
     logging.info("building tree...")
     tree = Tree.from_list(documents, root=root)
     logging.info("built tree: {}".format(tree))
